'use strict';

exports.__esModule = true;
exports.ensureValue = ensureValue;
exports.ensureListProperties = ensureListProperties;
exports.build = build;
exports.setMedia = setMedia;
exports.setReference = setReference;

var _react = require('react');

var _react2 = _interopRequireDefault(_react);

var _reactRedux = require('react-redux');

var _matchMedia = require('../matchMedia');

var _matchMedia2 = _interopRequireDefault(_matchMedia);

var _store = require('../store');

var _store2 = _interopRequireDefault(_store);

var _media = require('../reducers/media');

var _constants = require('../constants');

var _invariant = require('../utils/invariant');

var _invariant2 = _interopRequireDefault(_invariant);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

var optionsShape = _react.PropTypes.shape({
  columns: _react.PropTypes.number,
  gutter: _react.PropTypes.number,
  margin: _react.PropTypes.number,
  deaf: _react.PropTypes.bool,
  list: _react.PropTypes.arrayOf(_react.PropTypes.shape({
    name: _react.PropTypes.string.isRequired,
    query: _react.PropTypes.string.isRequired,
    gutter: _react.PropTypes.number,
    margin: _react.PropTypes.number
  }))
});

function ensureValue(options, base, key, value) {
  if (process.env.NODE_ENV !== 'production') {
    (0, _invariant2.default)(key, '<Grid> -> ensureValue -> key must be defined.');

    (0, _invariant2.default)(base, '<Grid> -> ensureValue -> base must be defined.');

    (0, _invariant2.default)(typeof base[key] !== 'undefined', '<Grid> -> ensureValue -> base -> key must be defined.');
  }

  if (value >= 0) return value;
  var result = options && options[key] >= 0 ? options[key] : base[key];
  return result;
}

function ensureListProperties(options, base, list) {
  return list.map(function (n) {
    var name = n.name;
    var query = n.query;
    var gutter = n.gutter;
    var margin = n.margin;

    return {
      name: name,
      query: query,
      gutter: ensureValue(options, base, 'gutter', gutter),
      margin: ensureValue(options, base, 'margin', margin)
    };
  });
}

function build() {
  var options = arguments.length <= 0 || arguments[0] === undefined ? {} : arguments[0];
  var base = arguments.length <= 1 || arguments[1] === undefined ? {} : arguments[1];
  var columns = options.columns;
  var _options$deaf = options.deaf;
  var deaf = _options$deaf === undefined ? false : _options$deaf;
  var _options$list = options.list;
  var list = _options$list === undefined ? base.list : _options$list;

  var size = list.length;

  (0, _invariant2.default)(!!size, '<Grid> -> options -> list can not be empty');

  if (columns) {
    (0, _invariant2.default)(!(columns % size) > 0, '<Grid> -> options -> columns must be a multiple of ' + '<Grid> -> options -> list -> length');
  }

  return {
    columns: columns || size * 4,
    deaf: deaf,
    list: ensureListProperties(options, base, list)
  };
}

function setMedia(name) {
  return { name: name };
}

function setReference(options) {
  return { options: options };
}

var Grid = function (_Component) {
  _inherits(Grid, _Component);

  function Grid(props, context) {
    _classCallCheck(this, Grid);

    // Initialize a new Model:
    // If <Grid> -> options is missing, it return a default Model.
    // if <Grid> -> options is provided, it return a valid Model.

    var _this = _possibleConstructorReturn(this, _Component.call(this, props, context));

    var model = build(props.options, _constants.MEDIA_MODEL_HELPER);

    _this.match = (0, _matchMedia2.default)(model.list);
    _this.shouldSubscribe = model.deaf !== true;

    // Initialize Redux `store`.
    var media = setMedia(_this.match.getCurrentName);
    var reference = setReference(model);
    _this.store = (0, _store2.default)({ media: media, reference: reference });
    return _this;
  }

  Grid.prototype.componentDidMount = function componentDidMount() {
    this.trySubscribe();
  };

  Grid.prototype.componentWillUnmount = function componentWillUnmount() {
    this.tryUnsubscribe();
  };

  Grid.prototype.render = function render() {
    return _react2.default.createElement(
      _reactRedux.Provider,
      { store: this.store },
      _react.Children.only(this.props.children)
    );
  };

  Grid.prototype.trySubscribe = function trySubscribe() {
    if (this.shouldSubscribe && !this.unsubscribe) {
      this.unsubscribe = this.match.subscribe(this.handleChange.bind(this));
    }
  };

  Grid.prototype.tryUnsubscribe = function tryUnsubscribe() {
    if (this.unsubscribe) {
      this.unsubscribe();
      this.unsubscribe = null;
    }
  };

  Grid.prototype.handleChange = function handleChange(payload) {
    if (!this.unsubscribe) {
      return;
    }
    this.store.dispatch((0, _media.updateMediaName)(payload));
  };

  return Grid;
}(_react.Component);

Grid.propTypes = {
  options: optionsShape,
  children: _react.PropTypes.element.isRequired
};
exports.default = Grid;