'use strict';

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

exports.__esModule = true;
exports.generateKey = exports.memoizeProcess = exports.getInReference = exports.reducePayload = exports.generatePayload = exports.listReducer = exports.parser = undefined;
exports.default = pick;

var _compact = require('lodash/array/compact');

var _compact2 = _interopRequireDefault(_compact);

var _get = require('lodash/object/get');

var _get2 = _interopRequireDefault(_get);

var _memoize = require('./memoize');

var _memoize2 = _interopRequireDefault(_memoize);

var _invariant = require('./invariant');

var _invariant2 = _interopRequireDefault(_invariant);

var _capitalize = require('./capitalize');

var _capitalize2 = _interopRequireDefault(_capitalize);

var _constants = require('../constants');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var parser = exports.parser = function parser(initial, input) {
  if (input && input.trim()) {
    return [initial].concat(input.trim().split(/\s+/));
  }
  return [initial];
};

var listReducer = exports.listReducer = function listReducer(name) {
  var list = arguments.length <= 1 || arguments[1] === undefined ? [] : arguments[1];

  return (0, _compact2.default)(list.map(function (n) {
    var _n$split = n.split('-');

    var entry = _n$split[0];

    var value = _n$split.slice(1);

    switch (value.length) {
      case 0:
        return entry;
      case 1:
        if (entry === 'offset') {
          return [entry].concat(value);
        }
        if (entry !== name) return false;
        return value[0];
      case 2:
        if (entry !== name) return false;
        if (value[0] === 'offset') {
          return value;
        }
        return false;
      default:
        return false;
    }
  }));
};

var generatePayload = exports.generatePayload = function generatePayload(_ref, list) {
  var name = _ref.name;

  return {
    name: name,
    list: listReducer(name, list)
  };
};

var reducePayload = exports.reducePayload = function reducePayload(_ref2, reference) {
  var name = _ref2.name;
  var list = _ref2.list;

  return list.reduce(function (acc, current) {
    var style = (0, _get2.default)(reference, [name].concat(current));
    return _extends({}, acc, style);
  }, {});
};

var getInReference = exports.getInReference = function getInReference(tag, _ref3) {
  var media = _ref3.media;
  var reference = _ref3.reference;
  var is = _ref3.is;

  var list = parser(tag, is);
  var payload = generatePayload(media, list);

  if (process.env.NODE_ENV !== 'production') {
    payload.list.forEach(function (n) {
      var value = Array.isArray(n) ? n[0] : n;
      (0, _invariant2.default)(_constants.WHITE_LIST[tag].indexOf(value) > -1, 'Property \'' + value + '\' is not allowed for <' + (0, _capitalize2.default)(tag) + '> component.');
    });
  }

  return reducePayload(payload, reference);
};

var memoizeProcess = exports.memoizeProcess = function memoizeProcess() {
  for (var _len = arguments.length, arg = Array(_len), _key = 0; _key < _len; _key++) {
    arg[_key] = arguments[_key];
  }

  return (0, _memoize2.default)(function () {
    return getInReference.apply(undefined, arg);
  });
};

var generateKey = exports.generateKey = function generateKey(tag, _ref4) {
  var name = _ref4.media.name;
  var is = _ref4.is;
  return '' + tag + name + is;
};

function pick() {
  var key = generateKey.apply(undefined, arguments);

  return memoizeProcess.apply(undefined, arguments)(key);
}